package model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

/**
 * Třída pro metodu latentní Dirichletovy alokace vypočítanou pomocí Gibbsova
 * vzorkování
 *
 * @author Matěj Fries
 */
public class Lda
{

    /**
     * Pomocné řídící proměnné pro algoritmus
     */
    private final double alpha = 0.01;
    private final double beta = 0.01;
    /**
     * celkové počty témat, dokumentů a slov
     */
    private final int pocT, pocD;
    private int pocS = 0;

    /**
     * Matice pro závislosti dokumentů na tématech a slov na tématech
     */
    private final Matice dokumentyTemata, slovaTemata;
    /**
     * List listů slov tvořících dokumenty a jejich statusů
     */
    private final List<ArrayList<String>> dokumenty = new ArrayList<>();
    private final List<ArrayList<Integer>> statusy = new ArrayList<>();
    private final Map<Integer, Double> nT = new HashMap<>();
    /**
     * Mapy indexů slov
     */
    private final Map<String, Integer> slovaIndexy = new HashMap<>();
    private final Map<Integer, String> indexySlova = new HashMap<>();

    /**
     * Konstruktor algoritmu, který si vytvoří potřebné mapy a matice a provede
     * algoritmus
     *
     * @param pocetTemat počet témat
     * @param slovaDokumentu list listů slov jednotlivých dokumentů
     */
    public Lda(int pocetTemat, List<List<String>> slovaDokumentu)
    {
        pocT = pocetTemat;

        for (int i = 0; i < slovaDokumentu.size(); i++)
        {
            ArrayList<String> dokument = new ArrayList<>();
            ArrayList<Integer> status = new ArrayList<>();
            for (int s = 0; s < slovaDokumentu.get(i).size(); s++)
            {
                String slovo = slovaDokumentu.get(i).get(s);

                dokument.add(slovo);
                status.add(-1);
                if (!slovaIndexy.containsKey(slovo))
                {
                    slovaIndexy.put(slovo, pocS);
                    indexySlova.put(pocS, slovo);
                    pocS++;
                }
            }
            dokumenty.add(dokument);
            statusy.add(status);
        }

        for (int i = 0; i < pocetTemat; i++)
        {
            this.nT.put(i, 0.0);
        }

        pocD = dokumenty.size();

        dokumentyTemata = new Matice(pocT, pocD);
        slovaTemata = new Matice(pocS, pocT);

        for (int i = 1; i <= 500; i++)
        {
            for (int d = 0; d < this.dokumenty.size(); d++)
            {
                gibbsuvVzorkovac(d);
            }
        }
    }

    /**
     * Privátní metoda pro gibbsovo vzorkování
     *
     * @param dok vybraný dokument pro vzorkování
     */
    private void gibbsuvVzorkovac(int dok)
    {
        Random nahoda = new Random();
        ArrayList<String> dokument = dokumenty.get(dok);
        ArrayList<Integer> status = statusy.get(dok);
        int i = 0;
        for (String slovo : dokument)
        {
            int stat = status.get(i);
            int s = slovaIndexy.get(slovo);

            double u = nahoda.nextDouble();
            double[] prav = new double[pocT + 1];
            prav[0] = 0.0;
            for (int t = 1; t <= pocT; t++)
            {
                double dt = dokumentyTemata.get(t - 1, dok);
                double st = slovaTemata.get(s, t - 1);

                double n = nT.get(t - 1);

                if (stat == (t - 1))
                {
                    dt -= 1.0;
                    st -= 1.0;
                    n -= 1.0;
                }
                prav[t] = prav[t - 1] + (dt + alpha) * (st + beta) / (n + pocS * beta);
            }

            for (int t = 1; t <= pocT; t++)
            {
                if (u <= prav[t] / prav[pocT])
                {
                    status.set(i, t - 1);

                    double novDt = dokumentyTemata.get(t - 1, dok);
                    double novSt = slovaTemata.get(s, t - 1);
                    double novN = nT.get(t - 1);
                    if (stat != (t - 1))
                    {
                        dokumentyTemata.set(t - 1, dok, novDt + 1.0);
                        slovaTemata.set(s, t - 1, novSt + 1.0);
                        nT.put(t - 1, (novN + 1.0));

                        if (stat >= 0)
                        {
                            double starDt = dokumentyTemata.get(stat, dok);
                            double starSt = slovaTemata.get(s, stat);

                            double starN = nT.get(stat);

                            dokumentyTemata.set(stat, dok, starDt - 1.0);
                            slovaTemata.set(s, stat, starSt - 1.0);
                            nT.put(stat, (starN - 1.0));
                        }
                    }
                    break;
                }
            }
            i++;
        }

        statusy.set(dok, status);
    }

    /**
     * Metoda pro získání závislostí dokumentů na tématech
     *
     * @return matici závislostí dokumentů na tématech
     */
    public Matice getDokumentyTemata()
    {
        return dokumentyTemata;
    }

    /**
     * Metoda pro získání vektorů dokumentů
     *
     * @return pole vektorů dokumentů
     */
    public Vektor[] getVektoryDokumentu()
    {
        return dokumentyTemata.getPoleVektoru(Matice.SLOUPCE);
    }
}
