package model;

import Jama.Matrix;

/**
 * Třídapro reprezentaci dvojrozměrného pole jako matice vycházející ze třídy
 * Matrix knihovny Jama
 *
 * @author Matěj Fries
 */
public class Matice extends Matrix
{

    /**
     * Konstanta pro přepínání metod na sloupce
     */
    public static final int SLOUPCE = 1;
    /**
     * Konstanta pro přepínání metod na řádky
     */
    public static final int RADKY = 2;

    /**
     * Konstruktor matice pomocí dvojrozměrného pole
     *
     * @param pole dvojrozměrné pole double prvků
     */
    public Matice(double[][] pole)
    {
        super(pole);
    }

    /**
     * Konstruktor, vytvoří matici o daných rozměrech naplněnou nulovými
     * hodnotami
     *
     * @param radky počet řádků matice
     * @param sloupce počet sloupců matice
     */
    public Matice(int radky, int sloupce)
    {
        super(radky, sloupce);
    }

    /**
     * Konstruktor, vytvoří matici o daných rozměrech a naplní ji zadanou
     * hodnotou
     *
     * @param radky počet řádků matice
     * @param sloupce počet sloupců matice
     * @param hodnota hodnota k naplnění matice
     */
    public Matice(int radky, int sloupce, double hodnota)
    {
        super(radky, sloupce, hodnota);
    }

    /**
     * Konstruktor vytvoří matici o daném počtu řádků a sloupců a naplní ji
     * hodnotami z pole
     *
     * @param pole hodnoty k naplnění matice
     * @param radky počet řádků
     * @param sloupce počet sloupců
     */
    public Matice(double[][] pole, int radky, int sloupce)
    {
        super(pole, radky, sloupce);
    }

    /**
     * Metoda pro transponování matice
     *
     * @return vrací novou transponovanou matici
     */
    @Override
    public Matice transpose()
    {
        return matrixNaMatici(maticeNaMatrix(this).transpose());
    }

    /**
     * Metoda pro vynásobení matice druhou maticí
     *
     * @param mat druhá matice k násobení
     * @return vrací novou vynásobenou matici
     */
    public Matice times(Matice mat)
    {
        return matrixNaMatici(maticeNaMatrix(this).times(maticeNaMatrix(mat)));
    }

    /**
     * Metoda pro zaokrouhlení hodnot v matici
     *
     * @param pocetMist počet desetinných míst, na které má být matice
     * zaokrouhlena
     * @return vrací novou zaokrouhlenou matici
     */
    public Matice zaokrouhliMatici(int pocetMist)
    {
        double[][] hodnoty = this.getArrayCopy();

        for (int x = 0; x < hodnoty.length; x++)
        {
            for (int y = 0; y < hodnoty[0].length; y++)
            {
                hodnoty[x][y] = Vektor.zaokrouhli(hodnoty[x][y], pocetMist);
            }
        }

        return new Matice(hodnoty);
    }

    /**
     * Metoda pro vytvoření pole vektorů z matice
     *
     * @param sloupceRadky přepínač pro zvolení, jeslti vektory budou sloupcové
     * nebo řádkové
     * @return vrací pole vektorů z matice
     */
    public Vektor[] getPoleVektoru(int sloupceRadky)
    {
        Vektor[] vektory = null;
        if (sloupceRadky == RADKY)
        {
            vektory = new Vektor[this.getRowDimension()];
            for (int x = 0; x < this.getRowDimension(); x++)
            {
                double[] vekPrvky = new double[this.getColumnDimension()];
                for (int y = 0; y < this.getColumnDimension(); y++)
                {
                    vekPrvky[y] = this.get(x, y);
                }
                vektory[x] = new Vektor(vekPrvky);
            }
        } else if (sloupceRadky == SLOUPCE)
        {
            vektory = new Vektor[this.getColumnDimension()];
            for (int y = 0; y < this.getColumnDimension(); y++)
            {
                double[] vekPrvky = new double[this.getRowDimension()];
                for (int x = 0; x < this.getRowDimension(); x++)
                {
                    vekPrvky[x] = this.get(x, y);
                }
                vektory[y] = new Vektor(vekPrvky);
            }
        }

        return vektory;
    }

    /**
     * Metoda pro vytvoření jednoho vektoru z matice
     *
     * @param sloupecRadek přepínač, jeslti bude vektor řádkový či sloupcový
     * @param kolikaty kolikátý sloupec nebo řádek k vytvoření vektoru
     * @return vrací Vektor z daného sloupce či řádku matice
     */
    public Vektor getVektor(int sloupecRadek, int kolikaty)
    {
        if (sloupecRadek == RADKY)
        {
            double[] vekPrvky = new double[this.getColumnDimension()];
            for (int y = 0; y < this.getColumnDimension(); y++)
            {
                vekPrvky[y] = this.get(kolikaty, y);
            }

            return new Vektor(vekPrvky);
        } else if (sloupecRadek == SLOUPCE)
        {
            double[] vekPrvky = new double[this.getRowDimension()];
            for (int x = 0; x < this.getRowDimension(); x++)
            {
                vekPrvky[x] = this.get(x, kolikaty);
            }

            return new Vektor(vekPrvky);
        }

        return null;
    }

    /**
     * Vytvoří matici z pole vektorů
     *
     * @param poleVektoru pole vektorů, ze kterých bude vytvořena matice
     * @param sloupceRadky přepínač, jeslti jsou vektory řádkové nebo sloupcové
     * @return vrací matici z pole vektorů
     */
    public static Matice MaticeZPoleVektoru(Vektor[] poleVektoru, int sloupceRadky)
    {
        double[][] prvky = null;
        if (sloupceRadky == RADKY)
        {
            prvky = new double[poleVektoru.length][poleVektoru[0].getPrvky().length];
            for (int x = 0; x < poleVektoru.length; x++)
            {
                for (int y = 0; y < poleVektoru[0].getPrvky().length; y++)
                {
                    prvky[x][y] = poleVektoru[x].getPrvek(y);
                }
            }
        } else if (sloupceRadky == SLOUPCE)
        {
            prvky = new double[poleVektoru[0].getPrvky().length][poleVektoru.length];
            for (int x = 0; x < poleVektoru[0].getPrvky().length; x++)
            {
                for (int y = 0; y < poleVektoru.length; y++)
                {
                    prvky[x][y] = poleVektoru[y].getPrvek(x);
                }
            }
        }

        return new Matice(prvky);
    }

    /**
     * Privátní metoda pro převedení třídy Matice na třídu Matrix
     *
     * @param m matice k převedení
     * @return vrací instanci třídy Matrix
     */
    private static Matrix maticeNaMatrix(Matice m)
    {
        return new Matrix(m.getArrayCopy());
    }

    /**
     * privátní metoda pro převedení třídy Matrix na třídu Matice
     *
     * @param m instrance třídy Matrix k převedení
     * @return vrací matici
     */
    private static Matice matrixNaMatici(Matrix m)
    {
        return new Matice(m.getArrayCopy());
    }

    /**
     * Vytvoří diagonální matici z vektoru
     *
     * @param v vektor s diagonálními hodnotami
     * @return vrací diagonální matici
     */
    public static Matice VektorNaDiagonalniMatici(Vektor v)
    {
        double[][] hodnoty = new double[v.getPrvky().length][v.getPrvky().length];
        for (int i = 0; i < v.getPrvky().length; i++)
        {
            for (int j = 0; j < v.getPrvky().length; j++)
            {
                hodnoty[i][j] = 0.0;
            }
            hodnoty[i][i] = v.getPrvek(i);
        }
        return new Matice(hodnoty);
    }
}
