package model;

/**
 * Třída PLSA, obsahuje matici obsahující TF hodnoty kolekce dokumentů, ze které
 * vypočítá matice pravděpodobností určitého počtu témat s pomocí TEM algoritmu
 *
 * @author Matěj Fries
 */
public class PLSA
{

    /**
     * Matice pro p(s|t), p(t|d), p(s|d) a TF nebo obdobné hodnoty
     */
    private final Matice pST, pTD, pSD, nDS;
    /**
     * Celkové počty témat, dokumentů a slov
     */
    private final int pocT, pocD, pocS;
    /**
     * Trojrozměrné pole pro latentní proměnnou p(t|d,s)
     */
    private final double[][][] pTDS;
    /**
     * Proměnná pro počítání počtu průběhů algoritmu
     */
    private final int pocetPrubehu;
    /**
     * Konstanta minimální rozdíl pro zastavení algoritmu při nepatrných změnách
     */
    private final double MINIMALNI_ROZDIL = 1e-6;
    /**
     * Proměnná beta pro TEM a její modifikátor
     */
    private final double beta, betaN;

    /**
     * Konstruktor třídy PLSA, který vypočítá pravděpodobnosti kolekce dokumentů
     *
     * @param matice vstupní hodnoty četnosti slov
     * @param pocetTemat počet témat
     * @param pocetPrubehu maximální počet průběhů algoritmu
     */
    public PLSA(Matice matice, int pocetTemat, int pocetPrubehu)
    {
        this.pocetPrubehu = pocetPrubehu;
        pocT = pocetTemat;
        pocD = matice.getRowDimension();
        pocS = matice.getColumnDimension();

        nDS = new Matice(matice.getArrayCopy());
        pSD = new Matice(pocD, pocS);
        pST = new Matice(pocS, pocT);
        pTD = new Matice(pocT, pocD);
        pTDS = new double[pocT][pocD][pocS];
        beta = 1.0;
        betaN = 0.8;
        
        double l, minL;
        double mistniBeta = this.beta;
        inicializace();

        eKrok(mistniBeta);
        l = mKrok(mistniBeta);

        for (int p = 0; p < 1; p++)
        {
            mistniBeta *= this.betaN;

            while (true)
            {
                eKrok(mistniBeta);
                minL = l;

                l = mKrok(mistniBeta);

                if (Math.abs(l - minL) / Math.abs(minL) < MINIMALNI_ROZDIL)
                {
                    break;
                }
            }
        }
    }

    /**
     * Privátní metoda pro inicializaci pravděpodobnostních matic náhodnými
     * hodnotami
     */
    private void inicializace()
    {

        // p(d,s)
        for (int d = 0; d < pocD; d++)
        {
            for (int s = 0; s < pocS; s++)
            {
                if (nDS.get(d, s) != 0)
                {
                    pSD.set(d, s, Math.random());
                }
            }
        }

        // p(d|z)
        for (int t = 0; t < pocT; t++)
        {
            for (int d = 0; d < pocD; d++)
            {
                pTD.set(t, d, Math.random());
            }
        }

        // p(s|t)
        for (int t = 0; t < pocT; t++)
        {
            for (int s = 0; s < pocS; s++)
            {
                pST.set(s, t, Math.random());
            }
        }

        // p(t|d,s)
        for (int t = 0; t < pocT; t++)
        {
            for (int d = 0; d < pocD; d++)
            {
                for (int s = 0; s < pocS; s++)
                {
                    if (nDS.get(d, s) != 0)
                    {
                        pTDS[t][d][s] = Math.random();
                    } else
                    {
                        pTDS[t][d][s] = 0;
                    }
                }
            }
        }
    }

    /**
     * Privátní metoda pro E krok TEM algoritmu
     *
     * @param beta beta parametr TEM algoritmu
     */
    private void eKrok(double beta)
    {
        for (int d = 0; d < pocD; d++)
        {
            for (int s = 0; s < pocS; s++)
            {
                if (nDS.get(d, s) != 0)
                {
                    for (int t = 0; t < pocT; t++)
                    {
                        pTDS[t][d][s] = Math.pow((pTD.get(t, d) * pST.get(s, t)), beta) * nDS.get(d, s) / pSD.get(d, s);
                    }
                }
            }
        }
    }

    /**
     * Privátní metoda pro M krok TEM algoritmu
     *
     * @param beta beta parametr TEM algoritmu
     * @return vrací logaritmickou pravděpodobnost kolekce dokumentů
     */
    private double mKrok(double beta)
    {

        double l = 0.0;
        // p(t|d)
        for (int d = 0; d < pocD; d++)
        {
            double soucet = 0.0;
            for (int t = 0; t < pocT; t++)
            {
                pTD.set(t, d, new Vektor(pTDS[t][d]).soucetVektoru());
                soucet += pTD.get(t, d);
            }
            for (int t = 0; t < pocT; t++)
            {
                pTD.set(t, d, pTD.get(t, d) / soucet);
            }
        }

        // p(s|t)
        for (int t = 0; t < pocT; t++)
        {
            double soucet = 0.0;
            for (int s = 0; s < pocS; s++)
            {
                double prav = 0.0;

                for (int d = 0; d < pocD; d++)
                {
                    prav += pTDS[t][d][s];
                }
                pST.set(s, t, prav);
                soucet += prav;
            }
            for (int s = 0; s < pocS; s++)
            {
                pST.set(s, t, pST.get(s, t) / soucet);
            }
        }

        //p(s|d) a l
        for (int d = 0; d < pocD; d++)
        {
            for (int s = 0; s < pocS; s++)
            {
                if (nDS.get(d, s) != 0)
                {
                    pSD.set(d, s, 0.0);
                    for (int t = 0; t < pocT; t++)
                    {
                        pSD.set(d, s, pSD.get(d, s) + Math.pow(pST.get(s, t) * pTD.get(t, d), beta));
                    }

                    l = l + nDS.get(d, s) * Math.log(pSD.get(d, s));
                }
            }
        }
        return l;

    }

    /**
     * Metoda pro získání matice p(s|t)
     *
     * @return vrací matici p(s|t)
     */
    public Matice getPST()
    {
        return pST;
    }

    /**
     * Metoda pro získání matice p(t|d)
     *
     * @return vrací matici p(t|d)
     */
    public Matice getPTD()
    {
        return pTD;
    }

    /**
     * Metoda pro získání vektorů dokumentů
     *
     * @return vrací pole vektorů, kde jeden vektor reprezentuje jeden dokument
     */
    public Vektor[] getVektoryDokumentu()
    {
        return pTD.getPoleVektoru(Matice.SLOUPCE);
    }
}
