package model;

import java.util.ArrayList;
import java.util.List;
import java.text.Collator;
import java.util.Locale;

/**
 * Třída pro soubor dokumentů a výpčet jejich četnosti slov
 *
 * @author Matěj Fries
 */
public class Soubor
{

    /**
     * List obsahující jednotlivé dokumenty obsažené v souboru
     */
    private final List<Dokument> dokumenty;
    /**
     * List všech slov vyskytujících se v dokumentech
     */
    private final List<String> slova = new ArrayList<>();
    /**
     * Matice obsahující hodnoty četnosti slov
     */
    private Matice mat;
    /**
     * Vektor obsahující celkové počty slov
     */
    private Vektor v;
    /**
     * Přepínač pro aktuálně vypočítaný formát četnost slov (TF nebo TF-IDF)
     */
    private int format = 0;
    /**
     * Přepínač pro výpočet celkového poču slov
     */
    private boolean celkovyPocet = false;
    /**
     * Přepínač pro výpočet počtu výskytů slov v dokumentech
     */
    private boolean pocetVyskytu = false;

    /**
     * Konstruktor souboru vytvoří seznam jedinečných slov
     *
     * @param dokumenty list dokumentů v kolekci
     */
    public Soubor(List<Dokument> dokumenty)
    {
        this.dokumenty = dokumenty;

        List<String> seznam;
        String slovo;

        for (int i = 0; i < dokumenty.size(); i++)
        {
            seznam = dokumenty.get(i).getSlova();

            for (int j = 0; j < seznam.size(); j++)
            {
                slovo = seznam.get(j);
                if (!slova.contains(slovo))
                {
                    slova.add(slovo);
                }
            }
        }

        mat = new Matice(slova.size(), dokumenty.size());
        v = new Vektor(new double[slova.size()]);
    }

    /**
     * Metoda pro vypočítání celkového počtu výskytů slov, který uloží do
     * parametru v
     */
    public void vypoctiCelkovyPocetVyskytuSlov()
    {
        if (!celkovyPocet)
        {
            double[] pocty = new double[slova.size()];
            for (int i = 0; i < slova.size(); i++)
            {
                double pocet = 0.0;
                String slovo = slova.get(i);
                for (int j = 0; j < dokumenty.size(); j++)
                {
                    if (dokumenty.get(j).getSlova().contains(slovo))
                    {
                        pocet += dokumenty.get(j).getPocet().get(dokumenty.get(j).getSlova().indexOf(slovo));
                    }
                }
                pocty[i] = pocet;
            }

            v = new Vektor(pocty);

            celkovyPocet = true;
            pocetVyskytu = false;
        }
    }

    /**
     * Metoda pro vypočítání početů výskytů slov v dokumentech, který uloží do
     * parametru v
     */
    public void vypoctiPocetVyskytuSlovVDokumentech()
    {
        if (!pocetVyskytu)
        {
            double[] pocty = new double[slova.size()];
            for (int i = 0; i < slova.size(); i++)
            {
                double pocet = 0.0;
                String slovo = slova.get(i);
                for (int j = 0; j < dokumenty.size(); j++)
                {
                    if (dokumenty.get(j).getSlova().contains(slovo))
                    {
                        pocet++;
                    }
                }
                pocty[i] = pocet;
            }

            v = new Vektor(pocty);

            celkovyPocet = false;
            pocetVyskytu = true;
        }
    }

    /**
     * Metoda pro vypočítání četnosti slov, kterou uloží do parametru mat
     */
    public void vypoctiTF()
    {
        if (format != 1)
        {
            for (int i = 0; i < slova.size(); i++)
            {
                String slovo = slova.get(i);
                for (int j = 0; j < dokumenty.size(); j++)
                {
                    if (dokumenty.get(j).getSlova().contains(slovo))
                    {
                        mat.set(i, j, dokumenty.get(j).getPocetDouble().get(dokumenty.get(j).getSlova().indexOf(slovo)));
                    } else
                    {
                        mat.set(i, j, 0.0);
                    }
                }
            }
            format = 1;
        }
    }

    /**
     * Metoda pro vypočítání TF-IDF hodnot, které uloží do parametru mat
     *
     * @param plusJedna přepínač, jestli se má k IDF v logaritmu přičíst
     * jednička
     */
    public void vypoctiIdf(boolean plusJedna)
    {
        if ((!plusJedna && format != 2) || (plusJedna && format != 3))
        {
            double pom;
            if (!pocetVyskytu)
            {
                this.vypoctiPocetVyskytuSlovVDokumentech();
            }

            for (int i = 0; i < slova.size(); i++)
            {
                String slovo = slova.get(i);
                for (int j = 0; j < dokumenty.size(); j++)
                {
                    if (dokumenty.get(j).getSlova().contains(slovo))
                    {
                        pom = dokumenty.get(j).getPocetDouble().get(dokumenty.get(j).getSlova().indexOf(slovo));
                        if (plusJedna)
                        {
                            pom *= Math.log10(dokumenty.size() / v.getPrvek(i) + 1);
                        } else
                        {
                            pom *= Math.log10(dokumenty.size() / v.getPrvek(i));
                        }
                        mat.set(i, j, pom);
                    } else
                    {
                        mat.set(i, j, 0.0);
                    }
                }
            }

            if (plusJedna)
            {
                format = 3;
            } else
            {
                format = 2;
            }
        }
    }

    /**
     * Metoda pro seřazení slov podle parametru v
     */
    public void seradPodleVektoru()
    {
        if (slova != null && slova.size() > 1 && (celkovyPocet || pocetVyskytu))
        {
            serad(0, slova.size() - 1, 1);
        }
    }

    /**
     * Metoda pro seřazení slov podle abecedy
     */
    public void seradPodleAbecedy()
    {
        if (slova != null && slova.size() > 1)
        {
            serad(0, slova.size() - 1, 2);
        }
    }

    /**
     * Privární metoda pro seřazení slov a počtů jejich výskytů
     *
     * @param l levý index
     * @param p pravý index
     * @param podle podle čeho se seznam řadí (1 - podle počtů, 2 - podle
     * abecedy)
     */
    private void serad(int l, int p, int podle)
    {
        Collator por = Collator.getInstance(new Locale("cs", "CZ"));
        por.setStrength(Collator.PRIMARY);

        if (l < p)
        {
            int b = l;
            for (int i = l + 1; i < p; i++)
            {
                switch (podle)
                {
                    case 1:
                        if (Double.compare(v.getPrvek(i), v.getPrvek(l)) > 0)
                        {
                            prohod(i, ++b);
                        }
                        break;
                    case 2:
                        if (por.compare(slova.get(i), slova.get(l)) < 0)
                        {
                            prohod(i, ++b);
                        }
                        break;
                    default:
                        break;
                }

            }
            prohod(l, b);
            serad(l, b, podle);
            serad(b + 1, p, podle);
        }
    }

    /**
     * Privátn metoda pro prohození dvou hodnot (prohodí list se slovy i s počty
     * výskytů)
     *
     * @param x první hodnota
     * @param y druhá hodnota
     */
    private void prohod(int x, int y)
    {
        String slovo = slova.get(x);
        slova.set(x, slova.get(y));
        slova.set(y, slovo);

        double pom = v.getPrvek(x);
        v.setPrvek(v.getPrvek(y), x);
        v.setPrvek(pom, y);

        if (format != 0)
        {
            mat = Vektor.prohodVektory(x, y, true, mat);
        }
    }

    /**
     * Metoda pro získání listu dokumentů
     *
     * @return vrací list dokumentů tvořící kolekci
     */
    public List<Dokument> getDokumenty()
    {
        return dokumenty;
    }

    /**
     * Metoda pro získání slov v kolekci
     *
     * @return vrací list slov
     */
    public List<String> getSlova()
    {
        return slova;
    }

    /**
     * Metoda pro získání pole vektorů dokumentů z parametru mat
     *
     * @return vrací vektory dokumentů
     */
    public Vektor[] getVektoryDokumentu()
    {
        return mat.getPoleVektoru(Matice.SLOUPCE);
    }

    /**
     * Metoda pro získání matice četnosti slov
     *
     * @return vrací parametr mat
     */
    public Matice getMat()
    {
        return mat;
    }

    /**
     * Metoda pro získání vektoru v obsahujícího celkový počet slov nebo počet
     * výskytů slov v dokumentech
     *
     * @return vrací parametr v
     */
    public Vektor getV()
    {
        return v;
    }

    /**
     * Metoda pro získání názvů dokumentů
     *
     * @return vrací pole stringů s názvy dokumentů
     */
    public String[] getNazvyDokumentu()
    {
        String[] nazvyDokumentu = new String[dokumenty.size()];
        for (int i = 0; i < dokumenty.size(); i++)
        {
            nazvyDokumentu[i] = getDokumenty().get(i).getNazev();
        }

        return nazvyDokumentu;
    }

    /**
     * Metoda pro zjištění, jestli je formát četnosti slov TF-IDF
     *
     * @param plusJedna varianta výpočetu IDF
     * @return vrací true, pokud ano a false, pokud ne
     */
    public boolean isIdf(boolean plusJedna)
    {
        if (plusJedna)
        {
            return (format == 3);
        } else
        {
            return (format == 2);
        }
    }

    /**
     * Metoda pro zjisštění, jestli formát četnosti slov je TF
     *
     * @return true pokud ano, jinak false
     */
    public boolean isTf()
    {
        return (format == 1);
    }

    //smazat
    public List<String> getPuvodniZneniDokumentu()
    {
        List<String> list = new ArrayList<>();
        for (int i = 0; i < dokumenty.size(); i++)
        {
            String dokument = "";
            for (int j = 0; j < dokumenty.get(i).getPuvodniZneni().size(); j++)
            {
                dokument += dokumenty.get(i).getPuvodniZneni().get(j) + " ";
            }
            list.add(dokument);
        }

        return list;
    }

    /**
     * Metoda pro získání vsutpních dat pro metodu LDA
     *
     * @return vrací list listů stringů obsahující původdní znění jednotlicých
     * dokumentů
     */
    public List<List<String>> getDokumentyProLDA()
    {
        List<List<String>> list = new ArrayList<>();

        for (int d = 0; d < dokumenty.size(); d++)
        {
            list.add(dokumenty.get(d).getPuvodniZneni());
        }

        return list;
    }
}
