package sluzby;

import Jama.SingularValueDecomposition;
import model.Lda;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import model.Dokument;
import model.Matice;
import model.SemantickaSVD;
import model.Soubor;
import model.PLSA;
import model.Vektor;

/**
 * Třída zajišťující obsluhu uživatelského rozhraní a dodávání dat pro zobrazení
 *
 * @author Matěj Fries
 */
public class SemantickaAnalyzaObsluha
{

    /**
     * Konstanta určující počet desetinných míst k zaokrouhlení
     */
    private static final int POCET_MIST_ZAOKROUHLENI = 5;

    /**
     * TableModel pro uživatelské rozhraní
     */
    private final TableModelSemAnalyza tableModel;

    /**
     * Jednotlivé metody pro reprezentaci kolekce dokumentů
     */
    private SemantickaSVD semSVD;
    private PLSA plsa;
    private Lda lda;
    private Soubor soubor;

    /**
     * Konstruktor třídy
     */
    public SemantickaAnalyzaObsluha()
    {
        tableModel = new TableModelSemAnalyza();
    }

    /**
     * Metoda pro načtení souborů
     *
     * @param importy pole souborů k načtení
     * @param zaokrouhli přepínač, určující zaokrouhlení dat
     * @return vrací true, pokud byly soubory úspěšně načteny, jinak false
     */
    public boolean nactiSoubor(File[] importy, boolean zaokrouhli)
    {
        List<String> soubory = new ArrayList<>();
        ImportSouboru imp = new ImportSouboru();

        for (File soub : importy)
        {
            if (!soub.getName().endsWith(".csv"))
            {
                return false;
            }
            soubory.add(soub.getAbsolutePath());
        }
        List<Dokument> dokumenty = new ArrayList<>();

        for (int i = 0; i < importy.length; i++)
        {
            try
            {
                dokumenty.add(new Dokument(imp.nacti(soubory.get(i), 2, ";"), importy[i].getName()));
                dokumenty.get(i).seradPodleAbecedy();

                this.soubor = new Soubor(dokumenty);
                this.soubor.seradPodleAbecedy();
                this.soubor.vypoctiTF();

                nastavVektorovouReprezentaci(zaokrouhli);

            } catch (UnsupportedEncodingException ex)
            {
                return false;
            } catch (FileNotFoundException ex)
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Privátní metoda pro vytvoření matice slov se slovy a četností slov
     *
     * @param zaokrouhli přepínač zaokrouhlení dat
     * @return vrací dvourozměrné pole slov
     */
    private String[][] maticeSeSlovy(boolean zaokrouhli)
    {
        Matice m = new Matice(soubor.getMat().getArrayCopy());
        if (zaokrouhli)
        {
            m = m.zaokrouhliMatici(POCET_MIST_ZAOKROUHLENI);
        }
        String[][] pole = new String[soubor.getMat().getRowDimension()][soubor.getMat().getColumnDimension() + 1];

        for (int x = 0; x < soubor.getMat().getRowDimension(); x++)
        {
            for (int y = 0; y < soubor.getMat().getColumnDimension() + 1; y++)
            {
                if (y == 0)
                {
                    pole[x][y] = soubor.getSlova().get(x);
                } else
                {
                    pole[x][y] = Double.toString(m.get(x, y - 1));
                }
            }
        }

        return pole;
    }

    /**
     * Privátní metoda pro získání názvů sloupců se slovy
     *
     * @return vrací pole slov
     */
    private String[] sloupceSeSlovy()
    {
        String[] sloupce = new String[soubor.getNazvyDokumentu().length + 1];

        sloupce[0] = "";
        for (int x = 0; x < soubor.getNazvyDokumentu().length; x++)
        {
            sloupce[x + 1] = soubor.getNazvyDokumentu()[x];
        }

        return sloupce;
    }

    /**
     * Privátní metoda pro vytvoření matice slov s celkovým početem výskytu slov
     *
     * @param mat matice slov, kte které se přidá celkový počet výskytů slov
     * @return dvourozměrné pole slov
     */
    public String[][] maticeSCelkem(String[][] mat)
    {

        String[][] pole = new String[mat.length][mat[0].length + 1];
        soubor.vypoctiCelkovyPocetVyskytuSlov();

        for (int x = 0; x < mat.length; x++)
        {
            for (int y = 0; y < mat[0].length + 1; y++)
            {
                if (y == mat[0].length)
                {
                    pole[x][y] = Double.toString(soubor.getV().getPrvek(x));
                } else
                {
                    pole[x][y] = mat[x][y];
                }
            }
        }

        return pole;
    }

    /**
     * Privátní metoda pro získání názvů sloupců s celkovým počtem slov
     *
     * @param sloup názvy sloupců
     * @return pole slov s názvy sloupců
     */
    private String[] sloupceSCelkem(String[] sloup)
    {
        String[] sloupce = new String[sloup.length + 1];

        for (int x = 0; x < sloup.length; x++)
        {
            sloupce[x] = sloup[x];
        }
        sloupce[sloupce.length - 1] = "Celkem";

        return sloupce;
    }

    /**
     * Metoda pro nastavení vektorové reprezentace
     *
     * @param zaokrouhli přepínač zaokrouhlení
     */
    public void nastavVektorovouReprezentaci(boolean zaokrouhli)
    {
        tableModel.setSloupce(sloupceSCelkem(sloupceSeSlovy()));
        tableModel.updatePole(maticeSCelkem(maticeSeSlovy(zaokrouhli)));
    }

    /**
     * Metoda pro nastavení LSA
     *
     * @param zaokrouhli přepínač zeokrouhlení
     */
    public void nastavSemSVD(boolean zaokrouhli)
    {
        semSVD = new SemantickaSVD(new SingularValueDecomposition(soubor.getMat()));

        tableModel.setSloupce(soubor.getNazvyDokumentu());
        if (zaokrouhli)
        {

            tableModel.updatePole(semSVD.getV().times(semSVD.getS()).transpose().zaokrouhliMatici(POCET_MIST_ZAOKROUHLENI));
        } else
        {
            tableModel.updatePole(semSVD.getV().times(semSVD.getS()).transpose());
        }
    }

    /**
     * Metoda pro nastavení LSA s určitým počtem témat
     *
     * @param pocetTemat počet témat
     * @param zaokrouhli přepínač zaokrouhlení
     */
    public void nastavSemSVD(int pocetTemat, boolean zaokrouhli)
    {
        semSVD = new SemantickaSVD(new SingularValueDecomposition(soubor.getMat()));
        semSVD = semSVD.zmensiKoncepty(pocetTemat);
        tableModel.setSloupce(soubor.getNazvyDokumentu());
        if (zaokrouhli)
        {
            tableModel.updatePole(semSVD.getV().times(semSVD.getS()).transpose().zaokrouhliMatici(POCET_MIST_ZAOKROUHLENI));
        } else
        {
            tableModel.updatePole(semSVD.getV().times(semSVD.getS()).transpose());
        }
    }

    /**
     * Metoda pro nastavení LDA
     *
     * @param zaokrouhli přepínaač zaokrouhlení
     */
    public void nastavLDA(boolean zaokrouhli)
    {
        nastavLDA(soubor.getDokumenty().size(), zaokrouhli);
    }

    /**
     * Metoda pro nastavení LDA s určitým počtem témat
     *
     * @param pocetTemat počet témat
     * @param zaokrouhli řepínač zaokrouhlení
     */
    public void nastavLDA(int pocetTemat, boolean zaokrouhli)
    {
        lda = new Lda(pocetTemat, soubor.getDokumentyProLDA());

        tableModel.setSloupce(soubor.getNazvyDokumentu());
        if (zaokrouhli)
        {
            tableModel.updatePole(lda.getDokumentyTemata().zaokrouhliMatici(POCET_MIST_ZAOKROUHLENI));
        } else
        {
            tableModel.updatePole(lda.getDokumentyTemata());
        }
    }

    /**
     * Metoda pro vypočítání podobnosti dvou dokumentů ve vektorové reprezentaci
     *
     * @param dokument1 číslo prvního dokumentu
     * @param dokument2 číslo druhého dokumentu
     * @param zaokrouhli přepínač zaokrouhlení
     * @return vrací cosinovu podobnost dokumentů
     */
    public double getPodobnostVek(int dokument1, int dokument2, boolean zaokrouhli)
    {
        if (zaokrouhli)
        {
            return Vektor.zaokrouhli(Vektor.cosinovaPodobnostVektoru(
                    soubor.getVektoryDokumentu()[dokument1],
                    soubor.getVektoryDokumentu()[dokument2]), POCET_MIST_ZAOKROUHLENI);
        } else
        {
            return Vektor.cosinovaPodobnostVektoru(
                    soubor.getVektoryDokumentu()[dokument1],
                    soubor.getVektoryDokumentu()[dokument2]);
        }
    }

    /**
     * Metoda pro vypočítání podobnosti dvou dokumentů v LSA
     *
     * @param dokument1 číslo prvního dokumentu
     * @param dokument2 číslo druhého dokumentu
     * @param zaokrouhli přepínač zaokrouhlení
     * @return vrací cosinovu podobnost dokumentů
     */
    public double getPodobnostLSA(int dokument1, int dokument2, boolean zaokrouhli)
    {
        if (zaokrouhli)
        {
            return Vektor.zaokrouhli(Vektor.cosinovaPodobnostVektoru(
                    semSVD.getVektoryDokumentu()[dokument1],
                    semSVD.getVektoryDokumentu()[dokument2]), POCET_MIST_ZAOKROUHLENI);
        }
        return Vektor.cosinovaPodobnostVektoru(
                semSVD.getVektoryDokumentu()[dokument1],
                semSVD.getVektoryDokumentu()[dokument2]);
    }

    /**
     * Metoda pro vypočítání podobnosti dvou dokumentů v PLSA
     *
     * @param dokument1 číslo prvního dokumentu
     * @param dokument2 číslo druhého dokumentu
     * @param zaokrouhli přepínač zaokrouhlení
     * @return vrací cosinovu podobnost dokumentů
     */
    public double getPodobnostPLSA(int dokument1, int dokument2, boolean zaokrouhli)
    {
        if (zaokrouhli)
        {
            return Vektor.zaokrouhli(Vektor.cosinovaPodobnostVektoru(
                    plsa.getVektoryDokumentu()[dokument1],
                    plsa.getVektoryDokumentu()[dokument2]), POCET_MIST_ZAOKROUHLENI);
        }
        return Vektor.cosinovaPodobnostVektoru(
                plsa.getVektoryDokumentu()[dokument1],
                plsa.getVektoryDokumentu()[dokument2]);
    }

    /**
     * Metoda pro vypočítání podobnosti dvou dokumentů v LDA
     *
     * @param dokument1 číslo prvního dokumentu
     * @param dokument2 číslo druhého dokumentu
     * @param zaokrouhli přepínač zaokrouhlení
     * @return vrací cosinovu podobnost dokumentů
     */
    public double getPodobnostLDA(int dokument1, int dokument2, boolean zaokrouhli)
    {
        if (zaokrouhli)
        {
            return Vektor.zaokrouhli(Vektor.cosinovaPodobnostVektoru(lda.getVektoryDokumentu()[dokument1],
                    lda.getVektoryDokumentu()[dokument2]), POCET_MIST_ZAOKROUHLENI);
        }
        return Vektor.cosinovaPodobnostVektoru(lda.getVektoryDokumentu()[dokument1],
                lda.getVektoryDokumentu()[dokument2]);
    }

    /**
     * Metoda pro nastavení PLSA
     *
     * @param zaokrouhli přepínač zaokrouhlení
     */
    public void nastavPLSA(boolean zaokrouhli)
    {
        nastavPLSA(soubor.getDokumenty().size(), zaokrouhli);
    }

    /**
     * Metoda pro nastavení PLSA s určitým počtem témat
     *
     * @param pocetTemat počet témat
     * @param zaokouhli přepínač zaokrouhlení
     */
    public void nastavPLSA(int pocetTemat, boolean zaokouhli)
    {
        plsa = new PLSA(soubor.getMat().transpose(), pocetTemat, 500);

        tableModel.setSloupce(soubor.getNazvyDokumentu());
        if (zaokouhli)
        {
            tableModel.updatePole(plsa.getPTD().zaokrouhliMatici(POCET_MIST_ZAOKROUHLENI));
        } else
        {
            tableModel.updatePole(plsa.getPTD());
        }

    }

    /**
     * Metoda pro nastavení TF-IDF
     *
     * @param plusJedna přepínač výpočetu IDF
     */
    public void nastavTfIDF(boolean plusJedna)
    {
        soubor.vypoctiIdf(plusJedna);
    }

    /**
     * Metoda pro nastavení TF
     */
    public void nastavTf()
    {
        soubor.vypoctiTF();
    }

    /**
     * Metoda pro získání kolekce dokumentů
     *
     * @return vrací soubor dokumentů
     */
    public Soubor getSoubor()
    {
        return soubor;
    }

    /**
     * Metoda pro získání tableModelu
     *
     * @return tableModel
     */
    public TableModelSemAnalyza getTableModel()
    {
        return tableModel;
    }
}
